unit LblWorks;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, RptWorks;

type TLabelType = (Avery5095, Avery5160, Avery5161, Avery5162,
                   Avery5163, Avery5164, Avery5165, Avery5196,
                   Avery5197, Avery5198, Avery5199, Avery5266,
                   Avery5267, Avery5293, Avery5294, Avery5295,
                   Avery5383, Avery5384, Avery5385, Avery5386,
                   Avery5388, Avery5389, Avery5395, Avery5660,
                   Avery5662, Avery5663, Custom);


type
  TLabelWorks = class(TComponent)
  private
    { Private declarations }
    Column:integer;
    FColumns:integer;
    FGap:single;
    FGridlines:Boolean;
    FHeight:single;
    FLabelFont:TFont;
    FLabelType:TLabelType;
    FLeftOffset:single;
    FLMargin:single;
    FMeasure:TUnits;
    FOutput:TReportWorks;
    FRows:integer;
    FSpacing:single;
    FStartLabel:integer;
    FTMargin:single;
    FTopOffset:single;
    FWidth:single;
    OldLineMeasure:TLineMeasure;
    OldUnit:TUnits;
    RoundLabel:boolean;
    Row:integer;
    procedure DrawGrid;
    procedure GetStartLabel;
    procedure SetFont;
    procedure SetLabelFont(Value: TFont);
    procedure SetMeasure(mValue:TUnits);
    procedure SetTabs;
    procedure SetType(lType:TLabelType);

  protected
    { Protected declarations }
  public
    { Public declarations }
    LabelNum:integer;
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    procedure DrawBox(x1,y1,x2,y2:single);
    procedure DrawEllipse(x1,y1,x2,y2:single);
    procedure DrawLine(x1,y1,x2,y2:single);
    procedure DrawPicture(x1,y1,x2,y2:single;APicture:TPicture);
    procedure EndLabels;
    procedure GoToTop;
    procedure NextLabel;
    procedure PrintLabelLine(s:string);
    procedure StartLabels;
  published
    { Published declarations }
    property Columns:integer read FColumns write FColumns;
    property Gap:single read FGap write FGap;
    property Gridlines:Boolean read FGridlines write FGridlines;
    property Height:single read FHeight write FHeight;
    property LabelFont:TFont read FLabelFont write SetLabelFont;
    property LabelType:TLabelType read FLabelType write SetType;
    property LeftMargin:single read FLMargin write FLMargin;
    property LeftOffset:single read FLeftOffset write FLeftOffset;
    property Measure:TUnits read FMeasure write SetMeasure;
    property Output:TReportWorks read FOutput write FOutput;
    property Rows:integer read FRows write FRows;
    property Spacing:single read FSpacing write FSpacing;
    property StartLabel:integer read FStartLabel write FStartLabel default 1;
    property TopMargin:single read FTMargin write FTMargin;
    property TopOffset:single read FTopOffset write FTopOffset;
    property Width:single read FWidth write FWidth;
  end;

procedure Register;

implementation

constructor TLabelWorks.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FLabelFont := TFont.Create;
  FLabelType := Custom;
  FStartLabel := 1;
end;


destructor TLabelWorks.Destroy;
begin
  FLabelFont.Free;
  inherited Destroy;
end;        


procedure TLabelWorks.SetLabelFont(Value: TFont);
begin
	FLabelFont.Assign(Value);
  SetFont;
end;


procedure TLabelWorks.StartLabels;
begin
  Row := 0;
  Column := 0;
  LabelNum := 1;
  with Output do
  begin
    {Set ReportWorks unit measure first.}
    OldUnit         := Units;
    Units           := Measure;
    OldLineMeasure  := LineMeasure;
    LineMeasure     := lmTextHeight;
    LeftMargin      := FLMargin;
    RightMargin     := FLMargin;
    TopMargin       := FTMargin;
    BottomMargin    := FTMargin;
    Startdoc;
    If Gridlines and (Destination = dPreview) then DrawGrid;
    end;
  SetTabs;
  GetStartLabel;
  SetFont;
end;

procedure TLabelWorks.EndLabels;
begin
  Output.EndDoc;
  {Reset ReportWorks properties.}
  Output.Units := OldUnit;
  Output.LineMeasure := OldLineMeasure;
end;

procedure TLabelWorks.SetFont;
begin
  Output.SetFont(LabelFont.Name, LabelFont.Size);
  Output.FontColor(LabelFont.Color);
  {Will need to rework this procedure and ReportWorks to
   make this work more efficently.}
  Output.FontRegular;
  if LabelFont.Style - [fsBold,fsStrikeout,fsUnderline] = [fsItalic]
     then Output.FontItalic;
  if LabelFont.Style - [fsItalic,fsBold,fsUnderline] = [fsStrikeout]
     then Output.FontStrikeout;
  if LabelFont.Style - [fsItalic,fsStrikeout,fsBold] = [fsUnderline]
     then Output.FontUnderline;
  if LabelFont.Style - [fsItalic,fsStrikeout,fsUnderline] = [fsBold]
     then Output.FontBold;
{ Output.ReportFont.Assign(LabelFont); }
end;

procedure TLabelWorks.PrintLabelLine(s:string);
begin
  Output.PrintTab(Column + 1, s);
  Output.NextLine;
end;


procedure TLabelWorks.SetTabs;
var x : integer;
begin
  for x := 0 to Columns - 1 do
    Output.SetTab((x * Width) + (x * Spacing) + LeftOffset, jLeft);
end;


procedure TLabelWorks.GoToTop;
begin
  Output.GoToXY(0,(Row * Height) + (Row * Gap) + TopOffset);
end;


procedure TLabelWorks.GetStartLabel;
var x:integer;
begin
  if StartLabel = 1 then GoToTop
  else for x := 2 to StartLabel do NextLabel;
end;


procedure TLabelWorks.NextLabel;
begin
  Inc(LabelNum);
  Inc(Column);
  if Column = Columns then
    begin
    Column := 0;
    Inc(Row);
    if Row = Rows then
      begin
      Row := 0;
      Output.NewPage;
      if GridLines and (Output.Destination = dPreview) then DrawGrid;
      end;
    end;
  GoToTop;
end;


procedure TLabelWorks.DrawBox(x1,y1,x2,y2:single);
begin
  x1 := x1 + (Column * Width) + (Column * Spacing);
  x2 := x2 + (Column * Width) + (Column * Spacing);
  y1 := y1 + (Row * Height) + (Row * Gap);
  y2 := y2 + (Row * Height) + (Row * Gap);
  Output.Rectangle(x1,y1,x2,y2);
  GoToTop;
end;


procedure TLabelWorks.DrawEllipse(x1,y1,x2,y2:single);
begin
  x1 := x1 + (Column * Width) + (Column * Spacing);
  x2 := x2 + (Column * Width) + (Column * Spacing);
  y1 := y1 + (Row * Height) + (Row * Gap);
  y2 := y2 + (Row * Height) + (Row * Gap);
  Output.Ellipse(x1,y1,x2,y2);
  GoToTop;
end;


procedure TLabelWorks.DrawLine(x1,y1,x2,y2:single);
begin
  x1 := x1 + (Column * Width) + (Column * Spacing);
  x2 := x2 + (Column * Width) + (Column * Spacing);
  y1 := y1 + (Row * Height) + (Row * Gap);
  y2 := y2 + (Row * Height) + (Row * Gap);
  Output.GoToXY(x1,y1);
  Output.LineTo(x2,y2);
  GoToTop;
end;

procedure TLabelWorks.DrawPicture(x1,y1,x2,y2:single;APicture:TPicture);
begin
  x1 := x1 + (Column * Width) + (Column * Spacing);
  x2 := x2 + (Column * Width) + (Column * Spacing);
  y1 := y1 + (Row * Height) + (Row * Gap);
  y2 := y2 + (Row * Height) + (Row * Gap);
  Output.StretchGraphic(x1,y1,x2,y2,APicture);
  GoToTop;
end;


procedure TLabelWorks.DrawGrid;
var x, row, column : integer;
    x1, y1 : single;
begin
  x1 := 0;
  y1 := 0;
  row := 0;
  column := 0;
  Output.SetPen(clSilver,2,psSolid,pmCopy);
  for x := 1 to (Rows * Columns) do
    begin
    if RoundLabel then
     Output.Ellipse(x1,y1,x1 + Width,y1 + Height) else
   { Output.Rectangle(x1, y1, x1+Width, y1+Height);}
     begin
     Output.GoToXY(x1, y1);
     Output.LineTo(x1 + Width, y1);
     Output.LineTo(x1 + Width, y1 + Height);
     Output.LineTo(x1, y1 + Height);
     Output.Lineto(x1, y1);
     end;
    Inc(column);
    if column = Columns then
      begin
      column := 0;
      Inc(row);
      end;
    x1 := (column * Width) + (column * Spacing);
    y1 := (row * Height) + (row * Gap);
    end;
  Output.SetPenDefault;
end;  


procedure TLabelWorks.SetMeasure(mValue:TUnits);
{Set measure and reset type values.}
begin
  FMeasure := mValue;
  SetType(FLabelType);    
end;


procedure TLabelWorks.SetType(lType:TLabelType);
begin
  FLabelType := lType;
  Gap        := 0;
  Spacing    := 0;
  Columns    := 2;
  RoundLabel := false;
  case lType of
    Avery5095 : begin
		FTMargin   := 0.583;
		FLMargin   := 0.688;
		Gap        := 0.167;
		Spacing    := 0.374;
		Rows       := 4;
		Height     := 2.323;
		Width      := 3.375;
		end;
    Avery5160 : begin
		FTMargin   := 0.5;
		FLMargin   := 0.188;
		Spacing    := 0.124;
		Rows       := 10;
		Columns    := 3;
		Height     := 1;
		Width      := 2.625;
		end;
    Avery5161 : begin
		FTMargin   := 0.5;
		FLMargin   := 0.149;
		Spacing    := 0.176;
		Rows       := 10;
		Height     := 1;
		Width      := 4;
		end;
    Avery5162 : begin
		FTMargin   := 0.833;
		FLMargin   := 0.156;
		Spacing    := 0.188;
		Rows       := 7;
		Height     := 1.332;
		Width      := 4;
		end;
    Avery5163 : begin
		FTMargin   := 0.5;
		FLMargin   := 0.2;
		Spacing    := 0.1;
		Rows       := 5;
		Height     := 2;
		Width      := 4;
		end;
    Avery5164 : begin
		FTMargin   := 0.5;
		FLMargin   := 0.175;
		Spacing    := 0.175;
		Rows       := 3;
		Height     := 3.333;
		Width      := 4;
		end;
    Avery5165 : begin
		FTMargin   := 0;
		FLMargin   := 0;
		Rows       := 1;
		Columns    := 1;
		Height     := 10.999;
		Width      := 8.499;
		end;
    Avery5196 : begin
		FTMargin   := 0.523;
		FLMargin   := 0.125;
		Gap        := 0.255;
		Rows       := 3;
		Columns    := 3;
		Height     := 2.75;
		Width      := 2.75;
		end;
    Avery5197 : begin
		FTMargin   := 1;
		FLMargin   := 0.166;
		Spacing    := 0.166;
		Rows       := 6;
		Height     := 1.5;
		Width      := 4;
		end;
    Avery5198 : begin
		FTMargin   := 0.5;
		FLMargin   := 0.5;
		Spacing    := 0.5;
		Rows       := 6;
		Height     := 1.666;
		Width      := 3.5;
		end;
    Avery5199 : begin
		FTMargin   := 0.917;
		FLMargin   := 1.069;
		Spacing    := 0.236;
		Rows       := 5;
		Height     := 1.833;
		Width      := 3.063;
		end;
    Avery5266 : begin
		FTMargin   := 0.5;
		FLMargin   := 0.531;
		Spacing    := 0.562;
		Rows       := 15;
		Height     := 0.666;
		Width      := 3.437;
		end;
    Avery5267 : begin
		FTMargin   := 0.5;
		FLMargin   := 0.312;
		Spacing    := 0.291;
		Rows       := 20;
		Columns    := 4;
		Height     := 0.5;
		Width      := 1.75;
		end;
    Avery5293 : begin
		FTMargin   := 0.509;
		FLMargin   := 0.432;
		Gap        := 0.038;
		Spacing    := 0.389;
		Rows       := 6;
		Columns    := 4;
		Height     := 1.638;
		Width      := 1.638;
		RoundLabel := true;
		end;
    Avery5294 : begin
		FTMargin   := 0.452;
		FLMargin   := 0.254;
		Gap        := 0.036;
		Spacing    := 0.254;
		Rows       := 4;
		Columns    := 3;
		Height     := 2.5;
		Width      := 2.5;
		RoundLabel := true;
		end;
    Avery5295 : begin
		FTMargin   := 0.472;
		FLMargin   := 0.629;
		Gap        := 0.025;
		Spacing    := 0.629;
		Rows       := 3;
		Height     := 3.333;
		Width      := 3.333;
		RoundLabel := true;
		end;
    Avery5383 : begin
		FTMargin   := 1.166;
		FLMargin   := 0.75;
		Rows       := 4;
		Height     := 2.166;
		Width      := 3.5;
		end;
    Avery5384 : begin
		FTMargin   := 1;
		FLMargin   := 0.25;
		Rows       := 3;
		Height     := 3;
		Width      := 4;
		end;
    Avery5385 : begin
		FTMargin   := 1.166;
		FLMargin   := 0.250;
		Rows       := 4;
		Height     := 2.166;
		Width      := 4;
		end;
    Avery5386 : begin
		FTMargin   := 1;
		FLMargin   := 1.75;
		Rows       := 3;
		Columns    := 1;
		Height     := 3;
		Width      := 5;
		end;
    Avery5388 : begin
		FTMargin   := 1;
		FLMargin   := 1.75;
		Rows       := 3;
		Columns    := 1;
		Height     := 3;
		Width      := 5;
		end;
    Avery5389 : begin
		FTMargin   := 1.5;
		FLMargin   := 1.25;
		Rows       := 2;
		Columns    := 1;
		Height     := 4;
		Width      := 6;
		end;
    Avery5395 : begin
		FTMargin   := 0.583;
		FLMargin   := 0.688;
		Gap        := 0.167;
		Spacing    := 0.374;
		Rows       := 4;
		Height     := 2.323;
		Width      := 3.375;
		end;
    Avery5660 : begin
		FTMargin   := 0.5;
		FLMargin   := 0;
		Rows       := 10;
		Columns    := 3;
		Height     := 1;
		Width      := 2.832;
		end;
    Avery5662 : begin
		FTMargin   := 0.834;
		FLMargin   := 0;
		Rows       := 7;
		Height     := 1.332;
		Width      := 4.25;
		end;
    Avery5663 : begin
		FTMargin   := 0.5;
		FLMargin   := 0;
		Rows       := 5;
		Height     := 2;
		Width      := 4.25;
		end;
  end;
  if Measure = uMM then
    begin
    FTMargin   := FTMargin * 2.42;
    FLMargin   := FLMargin * 2.42;
    Gap        := Gap * 2.42;
    Spacing    := Spacing * 2.42;
    Height     := Height * 2.42;
    Width      := Width * 2.42;
    end;  
  if Measure = uPoint then
    begin
    FTMargin   := FTMargin * 72;
    FLMargin   := FLMargin * 72;
    Gap        := Gap * 72;
    Spacing    := Spacing * 72;
    Height     := Height * 72;
    Width      := Width * 72;
    end;
end;


procedure Register;
begin
  RegisterComponents('Additional', [TLabelWorks]);
end;

end.
